# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: LoginEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import LogEvent_pb2 as LogEvent__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10LoginEvent.proto\x12\x11opentera.protobuf\x1a\x0eLogEvent.proto\"\x89\x07\n\nLoginEvent\x12/\n\nlog_header\x18\x01 \x01(\x0b\x32\x1b.opentera.protobuf.LogEvent\x12;\n\nlogin_type\x18\x02 \x01(\x0e\x32\'.opentera.protobuf.LoginEvent.LoginType\x12?\n\x0clogin_status\x18\x03 \x01(\x0e\x32).opentera.protobuf.LoginEvent.LoginStatus\x12\x13\n\x0b\x63lient_name\x18\x04 \x01(\t\x12\x16\n\x0e\x63lient_version\x18\x05 \x01(\t\x12\x11\n\tclient_ip\x18\x06 \x01(\t\x12\x0f\n\x07os_name\x18\x07 \x01(\t\x12\x12\n\nos_version\x18\x08 \x01(\t\x12\x11\n\tuser_uuid\x18\t \x01(\t\x12\x18\n\x10participant_uuid\x18\n \x01(\t\x12\x13\n\x0b\x64\x65vice_uuid\x18\x0b \x01(\t\x12\x14\n\x0cservice_uuid\x18\x0c \x01(\t\x12\x17\n\x0fserver_endpoint\x18\r \x01(\t\"\x85\x03\n\x0bLoginStatus\x12\x18\n\x14LOGIN_STATUS_UNKNOWN\x10\x00\x12\x18\n\x14LOGIN_STATUS_SUCCESS\x10\x01\x12+\n\'LOGIN_STATUS_FAILED_WITH_WRONG_PASSWORD\x10\x02\x12+\n\'LOGIN_STATUS_FAILED_WITH_WRONG_USERNAME\x10\x03\x12\x31\n-LOGIN_STATUS_FAILED_WITH_MAX_ATTEMPTS_REACHED\x10\x04\x12.\n*LOGIN_STATUS_FAILED_WITH_ALREADY_LOGGED_IN\x10\x05\x12*\n&LOGIN_STATUS_FAILED_WITH_INVALID_TOKEN\x10\x06\x12-\n)LOGIN_STATUS_FAILED_WITH_DISABLED_ACCOUNT\x10\x07\x12*\n&LOGIN_STATUS_FAILED_WITH_EXPIRED_TOKEN\x10\x08\"n\n\tLoginType\x12\x16\n\x12LOGIN_TYPE_UNKNOWN\x10\x00\x12\x14\n\x10LOGIN_TYPE_TOKEN\x10\x01\x12\x17\n\x13LOGIN_TYPE_PASSWORD\x10\x02\x12\x1a\n\x16LOGIN_TYPE_CERTIFICATE\x10\x03\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'LoginEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _LOGINEVENT._serialized_start=56
  _LOGINEVENT._serialized_end=961
  _LOGINEVENT_LOGINSTATUS._serialized_start=460
  _LOGINEVENT_LOGINSTATUS._serialized_end=849
  _LOGINEVENT_LOGINTYPE._serialized_start=851
  _LOGINEVENT_LOGINTYPE._serialized_end=961
# @@protoc_insertion_point(module_scope)
