# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: JoinSessionEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16JoinSessionEvent.proto\x12\x11opentera.protobuf\"\xed\x01\n\x10JoinSessionEvent\x12\x13\n\x0bsession_url\x18\x01 \x01(\t\x12\x1c\n\x14session_creator_name\x18\x02 \x01(\t\x12\x14\n\x0csession_uuid\x18\x03 \x01(\t\x12\x1c\n\x14session_participants\x18\x04 \x03(\t\x12\x15\n\rsession_users\x18\x05 \x03(\t\x12\x17\n\x0fsession_devices\x18\x06 \x03(\t\x12\x10\n\x08join_msg\x18\x07 \x01(\t\x12\x1a\n\x12session_parameters\x18\x08 \x01(\t\x12\x14\n\x0cservice_uuid\x18\t \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'JoinSessionEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _JOINSESSIONEVENT._serialized_start=46
  _JOINSESSIONEVENT._serialized_end=283
# @@protoc_insertion_point(module_scope)
