# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ServerCommand.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13ServerCommand.proto\x12\x11opentera.protobuf\x1a\x19google/protobuf/any.proto\"\xc3\x01\n\rServerCommand\x12:\n\x04type\x18\x01 \x01(\x0e\x32,.opentera.protobuf.ServerCommand.CommandType\x12\"\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any\"R\n\x0b\x43ommandType\x12\x0f\n\x0b\x43MD_UNKNOWN\x10\x00\x12\x17\n\x13\x43MD_REGISTER_DEVICE\x10\x01\x12\x19\n\x15\x43MD_CLIENT_DISCONNECT\x10\x02\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ServerCommand_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SERVERCOMMAND._serialized_start=70
  _SERVERCOMMAND._serialized_end=265
  _SERVERCOMMAND_COMMANDTYPE._serialized_start=183
  _SERVERCOMMAND_COMMANDTYPE._serialized_end=265
# @@protoc_insertion_point(module_scope)
