# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: UserEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fUserEvent.proto\x12\x11opentera.protobuf\"\x9a\x02\n\tUserEvent\x12\x11\n\tuser_uuid\x18\x01 \x01(\t\x12\x34\n\x04type\x18\x02 \x01(\x0e\x32&.opentera.protobuf.UserEvent.EventType\x12\x15\n\ruser_fullname\x18\x03 \x01(\t\"\xac\x01\n\tEventType\x12\x10\n\x0cUSER_UNKNOWN\x10\x00\x12\x12\n\x0eUSER_CONNECTED\x10\x01\x12\x15\n\x11USER_DISCONNECTED\x10\x02\x12\x10\n\x0cUSER_DELETED\x10\x03\x12\x0e\n\nUSER_ADDED\x10\x04\x12\x10\n\x0cUSER_UPDATED\x10\x05\x12\x17\n\x13USER_JOINED_SESSION\x10\x06\x12\x15\n\x11USER_LEFT_SESSION\x10\x07\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'UserEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _USEREVENT._serialized_start=39
  _USEREVENT._serialized_end=321
  _USEREVENT_EVENTTYPE._serialized_start=149
  _USEREVENT_EVENTTYPE._serialized_end=321
# @@protoc_insertion_point(module_scope)
