# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Result.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cResult.proto\x12\x11opentera.protobuf\"\xb2\x01\n\x06Result\x12\x32\n\x04type\x18\x01 \x01(\x0e\x32$.opentera.protobuf.Result.ResultType\x12\x0c\n\x04\x63ode\x18\x02 \x01(\x03\x12\x0f\n\x07message\x18\x03 \x01(\t\"U\n\nResultType\x12\x12\n\x0eRESULT_UNKNOWN\x10\x00\x12\r\n\tRESULT_OK\x10\x01\x12\x10\n\x0cRESULT_ERROR\x10\x02\x12\x12\n\x0eRESULT_WARNING\x10\x03\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'Result_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _RESULT._serialized_start=36
  _RESULT._serialized_end=214
  _RESULT_RESULTTYPE._serialized_start=129
  _RESULT_RESULTTYPE._serialized_end=214
# @@protoc_insertion_point(module_scope)
