# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: JoinSessionReplyEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bJoinSessionReplyEvent.proto\x12\x11opentera.protobuf\"\xb3\x02\n\x15JoinSessionReplyEvent\x12\x14\n\x0csession_uuid\x18\x01 \x01(\t\x12\x11\n\tuser_uuid\x18\x02 \x01(\t\x12\x18\n\x10participant_uuid\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65vice_uuid\x18\x04 \x01(\t\x12\x46\n\njoin_reply\x18\x05 \x01(\x0e\x32\x32.opentera.protobuf.JoinSessionReplyEvent.ReplyType\x12\x11\n\treply_msg\x18\x06 \x01(\t\"g\n\tReplyType\x12\x11\n\rREPLY_UNKNOWN\x10\x00\x12\x12\n\x0eREPLY_ACCEPTED\x10\x01\x12\x10\n\x0cREPLY_DENIED\x10\x02\x12\x0e\n\nREPLY_BUSY\x10\x03\x12\x11\n\rREPLY_TIMEOUT\x10\x04\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'JoinSessionReplyEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _JOINSESSIONREPLYEVENT._serialized_start=51
  _JOINSESSIONREPLYEVENT._serialized_end=358
  _JOINSESSIONREPLYEVENT_REPLYTYPE._serialized_start=255
  _JOINSESSIONREPLYEVENT_REPLYTYPE._serialized_end=358
# @@protoc_insertion_point(module_scope)
