# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: TeraModuleMessage.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17TeraModuleMessage.proto\x12\x11opentera.protobuf\x1a\x19google/protobuf/any.proto\"\xc6\x01\n\x11TeraModuleMessage\x12\x39\n\x04head\x18\x01 \x01(\x0b\x32+.opentera.protobuf.TeraModuleMessage.Header\x12\"\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32\x14.google.protobuf.Any\x1aR\n\x06Header\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x0c\n\x04time\x18\x02 \x01(\x01\x12\x0b\n\x03seq\x18\x03 \x01(\r\x12\x0e\n\x06source\x18\x04 \x01(\t\x12\x0c\n\x04\x64\x65st\x18\x05 \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'TeraModuleMessage_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TERAMODULEMESSAGE._serialized_start=74
  _TERAMODULEMESSAGE._serialized_end=272
  _TERAMODULEMESSAGE_HEADER._serialized_start=190
  _TERAMODULEMESSAGE_HEADER._serialized_end=272
# @@protoc_insertion_point(module_scope)
