# This file is automatically generated by cmake. Do not change.
opentera_server_version_string = "OpenTera_v1.2.5"
opentera_server_major_version = 1
opentera_server_minor_version = 2
opentera_server_patch_version = 5
