# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: LeaveSessionEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17LeaveSessionEvent.proto\x12\x11opentera.protobuf\"\x8d\x01\n\x11LeaveSessionEvent\x12\x14\n\x0csession_uuid\x18\x01 \x01(\t\x12\x14\n\x0cservice_uuid\x18\x02 \x01(\t\x12\x1c\n\x14leaving_participants\x18\x03 \x03(\t\x12\x15\n\rleaving_users\x18\x04 \x03(\t\x12\x17\n\x0fleaving_devices\x18\x05 \x03(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'LeaveSessionEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _LEAVESESSIONEVENT._serialized_start=47
  _LEAVESESSIONEVENT._serialized_end=188
# @@protoc_insertion_point(module_scope)
