# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ParticipantEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16ParticipantEvent.proto\x12\x11opentera.protobuf\"\x92\x03\n\x10ParticipantEvent\x12\x18\n\x10participant_uuid\x18\x01 \x01(\t\x12;\n\x04type\x18\x02 \x01(\x0e\x32-.opentera.protobuf.ParticipantEvent.EventType\x12\x18\n\x10participant_name\x18\x03 \x01(\t\x12 \n\x18participant_project_name\x18\x04 \x01(\t\x12\x1d\n\x15participant_site_name\x18\x05 \x01(\t\"\xcb\x01\n\tEventType\x12\x17\n\x13PARTICIPANT_UNKNOWN\x10\x00\x12\x19\n\x15PARTICIPANT_CONNECTED\x10\x01\x12\x1c\n\x18PARTICIPANT_DISCONNECTED\x10\x02\x12\x17\n\x13PARTICIPANT_DELETED\x10\x03\x12\x15\n\x11PARTICIPANT_ADDED\x10\x04\x12\x1e\n\x1aPARTICIPANT_JOINED_SESSION\x10\x05\x12\x1c\n\x18PARTICIPANT_LEFT_SESSION\x10\x06\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ParticipantEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _PARTICIPANTEVENT._serialized_start=46
  _PARTICIPANTEVENT._serialized_end=448
  _PARTICIPANTEVENT_EVENTTYPE._serialized_start=245
  _PARTICIPANTEVENT_EVENTTYPE._serialized_end=448
# @@protoc_insertion_point(module_scope)
