# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: LogEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0eLogEvent.proto\x12\x11opentera.protobuf\"\xa6\x02\n\x08LogEvent\x12\x33\n\x05level\x18\x01 \x01(\x0e\x32$.opentera.protobuf.LogEvent.LogLevel\x12\x11\n\ttimestamp\x18\x02 \x01(\x01\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x0f\n\x07message\x18\x04 \x01(\t\"\xb0\x01\n\x08LogLevel\x12\x14\n\x10LOGLEVEL_UNKNOWN\x10\x00\x12\x12\n\x0eLOGLEVEL_TRACE\x10\x07\x12\x12\n\x0eLOGLEVEL_DEBUG\x10\x06\x12\x11\n\rLOGLEVEL_INFO\x10\x05\x12\x14\n\x10LOGLEVEL_WARNING\x10\x04\x12\x15\n\x11LOGLEVEL_CRITICAL\x10\x03\x12\x12\n\x0eLOGLEVEL_ERROR\x10\x02\x12\x12\n\x0eLOGLEVEL_FATAL\x10\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'LogEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _LOGEVENT._serialized_start=38
  _LOGEVENT._serialized_end=332
  _LOGEVENT_LOGLEVEL._serialized_start=156
  _LOGEVENT_LOGLEVEL._serialized_end=332
# @@protoc_insertion_point(module_scope)
