# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: TeraEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fTeraEvent.proto\x12\x11opentera.protobuf\x1a\x19google/protobuf/any.proto\"\x9e\x01\n\tTeraEvent\x12\x33\n\x06header\x18\x01 \x01(\x0b\x32#.opentera.protobuf.TeraEvent.Header\x12$\n\x06\x65vents\x18\x02 \x03(\x0b\x32\x14.google.protobuf.Any\x1a\x36\n\x06Header\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x0c\n\x04time\x18\x02 \x01(\x01\x12\r\n\x05topic\x18\x03 \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'TeraEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TERAEVENT._serialized_start=66
  _TERAEVENT._serialized_end=224
  _TERAEVENT_HEADER._serialized_start=170
  _TERAEVENT_HEADER._serialized_end=224
# @@protoc_insertion_point(module_scope)
