# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: DeviceEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x44\x65viceEvent.proto\x12\x11opentera.protobuf\"\xcc\x02\n\x0b\x44\x65viceEvent\x12\x13\n\x0b\x64\x65vice_uuid\x18\x01 \x01(\t\x12\x36\n\x04type\x18\x02 \x01(\x0e\x32(.opentera.protobuf.DeviceEvent.EventType\x12\x13\n\x0b\x64\x65vice_name\x18\x03 \x01(\t\x12\x15\n\rdevice_status\x18\x04 \x01(\t\"\xc3\x01\n\tEventType\x12\x12\n\x0e\x44\x45VICE_UNKNOWN\x10\x00\x12\x14\n\x10\x44\x45VICE_CONNECTED\x10\x01\x12\x17\n\x13\x44\x45VICE_DISCONNECTED\x10\x02\x12\x12\n\x0e\x44\x45VICE_DELETED\x10\x03\x12\x10\n\x0c\x44\x45VICE_ADDED\x10\x04\x12\x19\n\x15\x44\x45VICE_JOINED_SESSION\x10\x05\x12\x17\n\x13\x44\x45VICE_LEFT_SESSION\x10\x06\x12\x19\n\x15\x44\x45VICE_STATUS_CHANGED\x10\x07\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'DeviceEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DEVICEEVENT._serialized_start=41
  _DEVICEEVENT._serialized_end=373
  _DEVICEEVENT_EVENTTYPE._serialized_start=178
  _DEVICEEVENT_EVENTTYPE._serialized_end=373
# @@protoc_insertion_point(module_scope)
