# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: DatabaseEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13\x44\x61tabaseEvent.proto\x12\x11opentera.protobuf\"\xbe\x01\n\rDatabaseEvent\x12\x38\n\x04type\x18\x01 \x01(\x0e\x32*.opentera.protobuf.DatabaseEvent.EventType\x12\x13\n\x0bobject_type\x18\x02 \x01(\t\x12\x14\n\x0cobject_value\x18\x03 \x01(\t\"H\n\tEventType\x12\x0e\n\nDB_UNKNOWN\x10\x00\x12\r\n\tDB_CREATE\x10\x01\x12\r\n\tDB_UPDATE\x10\x02\x12\r\n\tDB_DELETE\x10\x03\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'DatabaseEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DATABASEEVENT._serialized_start=43
  _DATABASEEVENT._serialized_end=233
  _DATABASEEVENT_EVENTTYPE._serialized_start=161
  _DATABASEEVENT_EVENTTYPE._serialized_end=233
# @@protoc_insertion_point(module_scope)
