# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: UserRegisterToEvent.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19UserRegisterToEvent.proto\x12\x11opentera.protobuf\"\xa9\x05\n\x13UserRegisterToEvent\x12\x44\n\nevent_type\x18\x01 \x01(\x0e\x32\x30.opentera.protobuf.UserRegisterToEvent.EventType\x12=\n\x06\x61\x63tion\x18\x02 \x01(\x0e\x32-.opentera.protobuf.UserRegisterToEvent.Action\"\xc2\x03\n\tEventType\x12\x11\n\rEVENT_UNKNOWN\x10\x00\x12\x18\n\x14\x45VENT_USER_CONNECTED\x10\x01\x12\x1b\n\x17\x45VENT_USER_DISCONNECTED\x10\x02\x12\x16\n\x12\x45VENT_USER_DELETED\x10\x03\x12\x14\n\x10\x45VENT_USER_ADDED\x10\x04\x12\x16\n\x12\x45VENT_USER_UPDATED\x10\x05\x12\x1f\n\x1b\x45VENT_PARTICIPANT_CONNECTED\x10\x06\x12\"\n\x1e\x45VENT_PARTICIPANT_DISCONNECTED\x10\x07\x12\x1d\n\x19\x45VENT_PARTICIPANT_DELETED\x10\x08\x12\x1b\n\x17\x45VENT_PARTICIPANT_ADDED\x10\t\x12\x1d\n\x19\x45VENT_PARTICIPANT_UPDATED\x10\n\x12\x1a\n\x16\x45VENT_DEVICE_CONNECTED\x10\x0b\x12\x1d\n\x19\x45VENT_DEVICE_DISCONNECTED\x10\x0c\x12\x18\n\x14\x45VENT_DEVICE_DELETED\x10\r\x12\x16\n\x12\x45VENT_DEVICE_ADDED\x10\x0e\x12\x18\n\x14\x45VENT_DEVICE_UPDATED\x10\x0f\"H\n\x06\x41\x63tion\x12\x12\n\x0e\x41\x43TION_UNKNOWN\x10\x00\x12\x13\n\x0f\x41\x43TION_REGISTER\x10\x01\x12\x15\n\x11\x41\x43TION_UNREGISTER\x10\x02\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'UserRegisterToEvent_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _USERREGISTERTOEVENT._serialized_start=49
  _USERREGISTERTOEVENT._serialized_end=730
  _USERREGISTERTOEVENT_EVENTTYPE._serialized_start=206
  _USERREGISTERTOEVENT_EVENTTYPE._serialized_end=656
  _USERREGISTERTOEVENT_ACTION._serialized_start=658
  _USERREGISTERTOEVENT_ACTION._serialized_end=730
# @@protoc_insertion_point(module_scope)
